@@ -186,7 +186,10 @@ impl<'a, I: Input> Lexer<'a, I> {
                         .read_number(true)
                         .map(|v| match v {
                             Left(v) => Num(v),
-                            Right(v) => BigInt(v),
+                            Right((value, raw)) => BigInt {
+                                value,
+                                raw: raw.into(),
+                            },
                         })
                         .map(Some);
                 }
@@ -269,7 +272,10 @@ impl<'a, I: Input> Lexer<'a, I> {
                             .read_number(false)
                             .map(|v| match v {
                                 Left(v) => Num(v),
-                                Right(v) => BigInt(v),
+                                Right((value, raw)) => BigInt {
+                                    value,
+                                    raw: raw.into(),
+                                },
                             })
                             .map(Some)
                     }
@@ -278,16 +284,23 @@ impl<'a, I: Input> Lexer<'a, I> {
                 return bigint
                     .map(|v| match v {
                         Left(v) => Num(v),
-                        Right(v) => BigInt(v),
+                        Right((value, raw)) => BigInt {
+                            value,
+                            raw: raw.into(),
+                        },
                     })
                     .map(Some);
             }
+
             '1'..='9' => {
                 return self
                     .read_number(false)
                     .map(|v| match v {
                         Left(v) => Num(v),
-                        Right(v) => BigInt(v),
+                        Right((value, raw)) => BigInt {
+                            value,
+                            raw: raw.into(),
+                        },
                     })
                     .map(Some)
             }
