@@ -102,7 +102,7 @@ impl<'a, I: Tokens> Parser<I> {
                     }
                 }
 
-                let type_parameters = p.parse_ts_type_params()?;
+                let type_parameters = p.parse_ts_type_params(false)?;
                 let mut arrow = p.parse_assignment_expr_base()?;
                 match *arrow {
                     Expr::Arrow(ArrowExpr {
